/************************************************************************/
/*                                                                      */
/* Program: laby.c                                                      */
/*                                                                      */
/* Labyrinth                                                            */
/* =========                                                            */
/*                                                                      */
/* This program calculates a labyrinth.                                 */
/* Image size: 512 x 512 pixels.                                        */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/* image: Output image name.                                            */
/* seed : Integer random seed (3).                                      */
/* msize: Size of path element in pixels (16).                          */
/* t    : Half thickness of path (t < msize/2) (7).                     */
/*                                                                      */
/*                                                                      */
/* Algorithm and program by: Stefan Spaennare, Lund, Sweden             */
/* E-mail:                   stefan@spaennare.se                        */
/*                                                                      */
/* First version:            June 1995                                  */
/* Latest update:            1998-08-22                                 */
/*                                                                      */
/************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "image.h"

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


int sgn(x)
int x;
{

   if (x == 0) {
      return(0);
   }
   else {
      if (x > 0) {
         return(1);
      }
      else {
         return(-1);
      } /* if */
   } /* if */
} /* sgn */



int main(argc,argv)
int argc;
char *argv[];
{
   struct image_head header;    
   short **image;

   double flops,ftime;

    int i,j,k,x1,y1,x,y,kx,ky,t,n,x2,y2,c,flag;
    int m,m2,f,g,xx1,xx2,yy1,yy2,k2,y01,y02,size,k3,dd;
    int f0,f1,f2,f3;
    int seed;
    int l2,x01,x02;
    int col,size2;

    long idum;

   clock_t time1,timediff;


   if (argc<4) {
      printf("Usage: %s image seedi msizei ti\n",argv[0]);
      exit(0);
   } /* if */

   seed=atoi(argv[2]);
   m=atoi(argv[3]);
   t=atoi(argv[4]);

   size2=512;

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=size2;
   header.wid=size2;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);

    f0=5000;
    f1=6000;
    f2=20000;
    f3=32767;

   for (i=0; i<size2; i++) {
      for (j=0; j<size2; j++) {
         image[i][j]=f3;
       } /* for j */
   } /* for i */


    m2=m/2;

    size=480;

   idum=-abs(seed)-1;

   g=(size-1-m)/m;
   g=g*m;

   for (i=0; i<=g+m; i++) {
      for (j=0; j<=g+m; j++) {
         image[i][j]=f0;
      } /* for j */
   } /* for i */


   for (i=0; i<=g+m; i++) {
      image[i][0]=f1;
      image[i][g+m]=f1;
      image[0][i]=f1;
      image[g+m][i]=f1;
   } /* for i */

    x=g/2;
    y=g/2;

    image[y][x]=f2;

    n=(size-2*m)/m;
    n=2*n*n-2;

    time1=clock();

for (j=1; j<=n; j++) {

   c=(int)(4.0*ran2(&idum));
   flag=0;

   i=0;
   while ((i <=3) && (flag==0)) {
      k=(i+c) % 4;

      if (k==0) {
	 if (image[y][x+m]==f0) {
	    x1=m; y1=0; flag=1;
	 } /* if */
      } /* if */
      if (k==1) {
	 if (image[y][x-m]==f0) {
	    x1=-m; y1=0; flag=1;
	 } /* if */
      } /* if */
      if (k==2) {
	 if (image[y+m][x]==f0) {
	    y1=m; x1=0; flag=1;
	 } /* if */
      } /* if */
      if (k==3) {
	 if (image[y-m][x]==f0) {
	    y1=-m; x1=0; flag=1;
	 } /* if */
      } /* if */
      i++;
   } /* while */

   if (flag==0) {

     if (image[y][x+m2]==f2) {
         x1=m; y1=0;
      }
      else if (image[y+m2][x]==f2) {
         y1=m; x1=0;
      }
      else if (image[y][x-m2]==f2) {
         x1=-m; y1=0;
      }
      else {
         y1=-m; x1=0;
      } /* if */

      f=f3;
   } 
   else {
      f=f2;
   } /* if */


    kx=sgn(x1)*t;
    ky=sgn(y1)*t;
    x2=x;
    y2=y;
    x=x+x1;
    y=y+y1;

    xx1=x2-kx+ky;
    yy1=y2+kx-ky;
    xx2=x+kx-ky;
    yy2=y-kx+ky;
       
    x01=xx1;
    x02=xx2;
    y01=yy1;
    y02=yy2;
       

    if (yy1 > yy2) {
       y02=yy1;
       y01=yy2;
    } /* if */

    if (xx1 > xx2) {
       x02=xx1;
       x01=xx2;
    } /* if */

    for (k2=y01; k2<=y02; k2++) {
       for (l2=x01; l2<=x02; l2++) {
          image[k2][l2]=f;
       } /* for l2 */
    } /* for k2 */

} /* for j */

timediff=clock()-time1;

ftime=(double)(timediff)/timetic;
printf("\nTime:  %12.2f\n",ftime);

WriteImage(argv[1],image,header,im_s16);

} /* End */
