/***********************************************************************/
/*                                                                     */
/* Program: tree.c                                                     */
/*                                                                     */
/* The algorithm is taken from the book:                               */
/*                                                                     */
/* "Fractals for the Classroom" part one, Springer Verlag by           */
/* H-O. Peitgen, H. Juurgens and D. Saupe.                             */
/*                                                                     */
/* Parameters:                                                         */
/*                                                                     */
/* image: Output image name.                                           */
/* size : Image size in pixels (512).                                  */
/* col  : Greyscale colour (0 <= col < 32768) (15000).                 */
/*                                                                     */
/*                                                                     */
/* Program by:    Stefan Spaennare, Lund, Sweden                       */
/* E-mail:        stefan@spaennare.se                                  */
/*                                                                     */
/* First version: June 1995                                            */
/* Latest update: 1998-08-22                                           */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "image.h"

double timetic=(double)(CLOCKS_PER_SEC);

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


int main(argc,argv)
int argc;
char *argv[];
{
   struct image_head header;    
   short **image;

   double flops,ftime,sfac,itfac;
   double left,w,wl,e1,e2,e3,e4,f1,f2,f3,f4,x,y,xn,yn,r,e5,f5;

   int i,j,itmax,it,size,xi,yi,col,seed;
   long idum;

   clock_t time1,timediff;


   if (argc<3) {
      printf("Usage: %s image sizei coli\n",argv[0]);
      exit(0);
   } /* if */

   size=atoi(argv[2]);
   col=atoi(argv[3]);
   

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=size;
   header.wid=size;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);


for (i=0; i<size; i++) {
   for (j=0; j<size; j++) {
       image[i][j]=32767;
    } /* for j */
} /* for i */

left=30.0;
w=200.0;
wl=w+left;

e1=0.4431*w; e2=0.2511*w; e3=0.5976*w; e4=0.4884*w; e5=0.8562*w;
f1=0.2452*w; f2=0.5692*w; f3=0.0969*w; f4=0.5069*w; f5=0.2513*w;

x=e1;
y=0.0;

itfac=(double)(size)/1024.0;
itmax=(int)(itfac*itfac*1000000.0+0.5); 

sfac=4.8*itfac;

seed=3;
idum=-abs(seed)-1;

time1=clock();

for (i=1; i<=itmax; i++) {

    r=ran2(&idum);

    if (r < 0.2) {
      xn=0.195*x-0.488*y+e1;
       yn=0.344*x+0.443*y+f1;
    } 
    else if (r < 0.4) {
       xn=0.462*x+0.414*y+e2;
       yn=-0.252*x+0.361*y+f2;
    }
    else if (r < 0.6) {
       xn=-0.058*x-0.070*y+e3;
       yn=0.453*x-0.111*y+f3;
    }
    else if (r < 0.8) {
       xn=-0.035*x+0.070*y+e4;
       yn=-0.469*x-0.022*y+f4;
    }
    else {
       xn=-0.637*x+0.0*y+e5;
       yn=0.0*x+0.501*y+f5;
    } /* if */

    xi=(int)(sfac*(xn+left-25.0)+0.5);
    yi=(int)(sfac*(wl-yn-20.0)+0.5);

    image[yi][xi]=col;


    x=xn;
    y=yn;

} /* for i */


timediff=clock()-time1;

ftime=(double)(timediff)/timetic;
printf("\nTime:  %12.2f\n",ftime);

WriteImage(argv[1],image,header,im_s16);

} /* End */
